<?php

class SendmailLogic extends SOY2LogicBase{
	
	private $config;
	private $userDao;
	private $itemOrderDao;
	private $itemDao;
	private $pageDao;
	
	function SendmailLogic(){
		SOY2::imports("module.plugins.order_later_sendmail.util.*");
		SOY2::import("domain.order.SOYShop_Order");
		$this->userDao = SOY2DAOFactory::create("user.SOYShop_UserDAO");
	}
	
	function execute(){
		
		$this->config = OrderLaterSendmailUtil::getConfig();
		
		//設定が無い場合は処理を終了
		if(!isset($this->config["date"])) return false;
		
		$users = $this->getUsersAndOrderIdInKey();
		
		if(count($users) === 0) return false;
		
		//取得したユーザ情報を元にメール送信の処理を開始する
		$title = OrderLaterSendmailUtil::getMailTitle();
		$content = OrderLaterSendmailUtil::getMailContent();
		$content = $this->convertCompanyInfomation($content);
		
		//MailLogicの呼び出し
		$mailLogic = SOY2Logic::createInstance("logic.mail.MailLogic");
		
		foreach($users as $orderId => $user){
			if(strlen($user->getMailAddress()) === 0) continue;
			$title = $this->convertMailTitle($title);
			$body = $this->convertMailContent($content, $user, $orderId);
	
			$order = new SOYShop_Order();
			$order->setId($orderId);
			$order->setUserId($user->getId());
			$mailLogic->sendMail($user->getMailAddress(), $title, $body, null, $order);
		}
	}
	
	function convertMailTitle($title){
		$title = $this->convertCompanyInfomation($title);
		return trim($title);
	}
		
	/**
	 * @params String content, object SOYShop_User, object SOYShop_Item item
	 * @return String body
	 */
	function convertMailContent($content, SOYShop_User $user, $orderId = null){
		//ユーザー情報
		$content = str_replace("#NAME#", $user->getName(), $content);
		$content = str_replace("#READING#", $user->getReading(), $content);
		$content = str_replace("#MAILADDRESS#", $user->getMailAddress(), $content);
		$content = str_replace("#BIRTH_YEAR#", $user->getBirthdayYear(), $content);
		$content = str_replace("#BIRTH_MONTH#", $user->getBirthdayMonth(), $content);
		$content = str_replace("#BIRTH_DAY#", $user->getBirthdayDay(), $content);
		
		//商品情報
		if(strpos($content, "#ORDER_ITEMS_LIST#") && isset($orderId)){
			$items = $this->getItemsByOrderId($orderId);
			if(count($items) > 0){
				$listTextArray = array();
				foreach($items as $item){
					$listTextArray[] = $item->getName();
					$listTextArray[] = $this->getDetailPageUrl($item);
				}
				
				$content = str_replace("#ORDER_ITEMS_LIST#", implode("\n", $listTextArray), $content);
			}
		}
		
		//最初に改行が存在した場合は改行を削除する
		return trim($content);
	}
	
	function getItemsByOrderId($orderId){
		if(!$this->itemOrderDao) $this->itemOrderDao = SOY2DAOFactory::create("order.SOYShop_ItemOrderDAO");
		if(!$this->itemDao) $this->itemDao = SOY2DAOFactory::create("shop.SOYShop_ItemDAO");
		
		$items = array();
		
		try{
			$itemOrders = $this->itemOrderDao->getByOrderId($orderId);
		}catch(Exception $e){
			$itemOrders = array();
		}
		
		if(count($itemOrders)){
			foreach($itemOrders as $itemOrder){
				$itemId = $itemOrder->getItemId();
				try{
					$items[] = $this->itemDao->getById($itemId);
				}catch(Exception $e){
					continue;
				}
			}
		}
		
		return $items;
	}
	
	function getDetailPageUrl($item){
		if(!$this->pageDao) $this->pageDao = SOY2DAOFactory::create("site.SOYShop_PageDAO");
		try{
			$page = $this->pageDao->getById($item->getDetailPageId());
		}catch(Exception $e){
			return null;
		}
		
		return soyshop_get_site_url(true) . $page->getUri() . "/" . $item->getAlias();
	}
	
	function convertCompanyInfomation($content){
		
		if(!$this->shopConfig){
			SOY2::import("domain.config.SOYShop_ShopConfig");
			$this->shopConfig = SOYShop_ShopConfig::load();
		}
		$config = $this->shopConfig;

		$content = str_replace("#SHOP_NAME#", $config->getShopName(), $content);

		$company = $config->getCompanyInformation();
		foreach($company as $key => $value){
			$content = str_replace(strtoupper("#COMPANY_" . $key ."#"), $value, $content);
		}
		$content = str_replace("#SITE_URL#", soyshop_get_site_url(true), $content);
		
		return $content;
	}
	
	/**
	 * 管理画面で設定した条件にマッチする顧客情報を取得する
	 * @return array(orderId => SOYShop_User);
	 */
	function getUsersAndOrderIdInKey(){
		$noticeDate = (int)$this->config["date"];
		
		//範囲を作成する
		$start = time() - ($noticeDate + 1) * 24 * 60 * 60;
		$end = time() - $noticeDate * 24 * 60 * 60;
		
		
		$sql = "SELECT o.id, o.user_id, user.mail_address, user.name, user.reading, user.birthday ".
				"FROM soyshop_order o ".
				"INNER JOIN soyshop_user user ".
				"ON o.user_id = user.id ".
				"WHERE o.order_date > " . $start . " ".
				"AND o.order_date < " . $end . " ".
				"AND o.order_status < " . SOYShop_Order::ORDER_STATUS_CANCELED;
		try{
			$results = $this->userDao->executeQuery($sql, array());
		}catch(Exception $e){
			return array();
		}
		
		$users = array();
		foreach($results as $result){
			$orderId = $result["id"];
			$result["id"] = $result["user_id"];
			$users[$orderId] = $this->userDao->getObject($result);
		}
		
		return $users;
	}
}
?>